# openITCOCKPIT Satellite Frontend Angular

**Work in Progress**

> [!CAUTION]
> This is Enterprise Code! Make sure your PhpStorm is set to add the openITCOCKPIT Enterprise license !!!

## Setup for Developers

The openITCOCKPIT Satellite frontend is a standalone Angular application that communicates with the [openITCOCKPIT Satellite backend](https://github.com/it-novum/openITCOCKPIT-Satellite-Frontend)
via the [HTTP API](https://docs.openitcockpit.io/en/development/api/).

If you want to contribute to the openITCOCKPIT Satellite frontend, you can follow the instructions below to set up a development environment.

### Prerequisites

- Latest Node.js LTS ([Ubuntu / Debian download](https://github.com/nodesource/distributions))
- [Angular CLI](https://angular.dev/installation) installed globally
   ```
   npm install -g @angular/cli
   ```

### System requirements

The openITCOCKPIT Angular application is a relatively large application and requires a lot of resources to build and run.
We recommend at least 8 GB of RAM and 4 CPU cores.

### Nginx Configuration

To avoid any CORS issues, the frontend should be installed on an openITCOCKPIT server. The location does not matter however
`/opt/openitc/frontend-angular` is the recommended location.

To route all traffic through the Nginx web server, edit the config file `/etc/nginx/openitc/satellite.conf`
and commend out the `Proxy for the Angular Frontend (production)` section and uncomment the `Proxy for the Angular Frontend (development)` section.

```nginx
# Proxy for the Angular Frontend (production)
#location ^~ /a/ {
#    alias /opt/openitc/frontend-angular/browser/;
#    index index.html;
#    try_files $uri $uri/ /a/index.html;
#}

# Proxy for the Angular Frontend (development)
location ^~ /a/ {
    proxy_pass http://127.0.0.1:4200/a/;

    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "Upgrade";

    proxy_set_header    Host                $host;
    proxy_set_header    X-Real-IP           $remote_addr;
    proxy_set_header    X-Forwarded-Host    $host;
    proxy_set_header    X-Forwarded-Server  $host;
    proxy_set_header    X-Forwarded-Proto   $scheme;
    proxy_set_header    X-Forwarded-For     $remote_addr;
    proxy_redirect off;
    proxy_connect_timeout 90s;
    proxy_read_timeout 90s;
    proxy_send_timeout 90s;
}
```

To apply the changes, restart the Nginx service.

```
systemctl restart nginx
```

### Clone the Repository

```
cd /opt/openitc/
git clone https://github.com/it-novum/openITCOCKPIT-Satellite-frontend-angular.git frontend-angular
```

You can now start the Angular development server.

```
cd /opt/openitc/frontend-angular
npm install
chown www-data:www-data /opt/openitc/frontend-angular -R
```

```
npm start
```

The frontend is now available at `https://xxx.xxx.xxx.xxx/a/`.

## Build Angular Satellite Frontend for Production

```
npm install
ng build
```

## Create a new module

1. Create a new directory at `src/app/modules`, for example `jira_module`.
2. Now add a basic routes configuration for the module `jira_module/jira_module.routes.ts`

```typescript
import { Routes } from '@angular/router';

export const jiraModuleRoutes: Routes = [];
```

3. Load the routs into the Angular app: `src/app/app.routes.ts`

````typescript
/***    Routes for modules   ***/
const moduleRoutes: Routes = [
        ...eventcorrelationModuleRoutes,
        ...jiraModuleRoutes // <-- add your module routes
    ];
````

4. Inside of `src/app/modules/jira_module` you can create different sub-folders for `pages`, `services`, etc.

## License

```
openITCOCKPIT Enterprise Edition

If you purchased an openITCOCKPIT Enterprise Edition you can use this file
under the terms of the openITCOCKPIT Enterprise Edition licence agreement.
Licence agreement and licence key will be shipped with the order
confirmation.
```

