import { Component, inject } from "@angular/core";
import { DOCUMENT } from "@angular/common";
import { ActivatedRoute, RouterModule, Routes } from '@angular/router';
import { authGuard } from "./auth/auth.guard";
import { mapModuleRoutes } from './modules/map_module/map_module.routes';

@Component({
    selector: 'legacy-redirect',
    template: `If you can read this, something has to be fixed ;)`,
    imports: [RouterModule]
})
class LegacyUrlComponent {
    private readonly route = inject(ActivatedRoute);
    private readonly document = inject(DOCUMENT);

    public constructor() {

        const {legacyUrl} = this.route.snapshot.data;

        if (legacyUrl) {

            console.log('destination', legacyUrl)
            this.document.location.href = legacyUrl;
        }
    }
}

/***    Routes for modules   ***/
const moduleRoutes: Routes = [
    ...mapModuleRoutes,
];
/***    Core routes   ***/
const coreRoutes: Routes = [{
    path: '',
    loadComponent: () => import('./pages/dashboards/dashboards-index/dashboards-index.component').then(m => m.DashboardsIndexComponent),
    canActivate: [authGuard]
}, {
    path: 'dashboards/index',
    loadComponent: () => import('./pages/dashboards/dashboards-index/dashboards-index.component').then(m => m.DashboardsIndexComponent)
}, {
    path: 'profile/edit',
    loadComponent: () => import('./pages/profile/profile-edit/profile-edit.component').then(m => m.ProfileEditComponent)
}, {
    path: 'automaps/index',
    loadComponent: () => import('./pages/automaps/automaps-index/automaps-index.component').then(m => m.AutomapsIndexComponent)
}, {
    path: 'users/login',
    loadComponent: () => import('./pages/users/users-login/users-login.component').then(m => m.UsersLoginComponent)
}, {
    path: 'users/index',
    loadComponent: () => import('./pages/users/users-index/users-index.component').then(m => m.UsersIndexComponent)
}, {
    path: 'users/add',
    loadComponent: () => import('./pages/users/users-add/users-add.component').then(m => m.UsersAddComponent)
}, {
    path: 'users/edit/:id',
    loadComponent: () => import('./pages/users/users-edit/users-edit.component').then(m => m.UsersEditComponent)
}, {
    path: 'usergroups/index',
    loadComponent: () => import('./pages/usergroups/usergroups-index/usergroups-index.component').then(m => m.UsergroupsIndexComponent)
}, {
    path: 'usergroups/add',
    loadComponent: () => import('./pages/usergroups/usergroups-add/usergroups-add.component').then(m => m.UsergroupsAddComponent)
}, {
    path: 'usergroups/edit/:id',
    loadComponent: () => import('./pages/usergroups/usergroups-edit/usergroups-edit.component').then(m => m.UsergroupsEditComponent)
}, {
    path: 'hosts/index',
    loadComponent: () => import('./pages/hosts/hosts-index/hosts-index.component').then(m => m.HostsIndexComponent)
}, {
    path: 'hosts/browser/:uuid',
    loadComponent: () => import('./pages/hosts/hosts-browser/hosts-browser.component').then(m => m.HostsBrowserComponent)
}, {
    path: 'services/index',
    loadComponent: () => import('./pages/services/services-index/services-index.component').then(m => m.ServicesIndexComponent)
}, {
    path: 'services/browser/:uuid',
    loadComponent: () => import('./pages/services/services-browser/services-browser.component').then(m => m.ServicesBrowserComponent)
}, {
    path: 'hostchecks/index/:uuid',
    loadComponent: () => import('./pages/hostchecks/hostchecks-index/hostchecks-index.component').then(m => m.HostchecksIndexComponent)
}, {
    path: 'servicechecks/index/:uuid',
    loadComponent: () => import('./pages/servicechecks/servicechecks-index/servicechecks-index.component').then(m => m.ServicechecksIndexComponent)
}, {
    path: 'statehistories/host/:uuid',
    loadComponent: () => import('./pages/statehistories/statehistories-host/statehistories-host.component').then(m => m.StatehistoriesHostComponent)
}, {
    path: 'statehistories/service/:uuid',
    loadComponent: () => import('./pages/statehistories/statehistories-service/statehistories-service.component').then(m => m.StatehistoriesServiceComponent)
}, {
    path: 'acknowledgements/host/:uuid',
    loadComponent: () => import('./pages/acknowledgements/acknowledgements-host/acknowledgements-host.component').then(m => m.AcknowledgementsHostComponent)
}, {
    path: 'acknowledgements/service/:uuid',
    loadComponent: () => import('./pages/acknowledgements/acknowledgements-service/acknowledgements-service.component').then(m => m.AcknowledgementsServiceComponent)
}, {
    path: 'downtimes/host/:uuid',
    loadComponent: () => import('./pages/downtimes/downtimes-host/downtimes-host.component').then(m => m.DowntimesHostComponent)
}, {
    path: 'downtimes/service/:uuid',
    loadComponent: () => import('./pages/downtimes/downtimes-service/downtimes-service.component').then(m => m.DowntimesServiceComponent)
}, {
    path: 'automaps/index',
    loadComponent: () => import('./pages/automaps/automaps-index/automaps-index.component').then(m => m.AutomapsIndexComponent)
}, {
    path: 'agentconnector/pull',
    loadComponent: () => import('./pages/agentconnector/agentconnector-pull/agentconnector-pull.component').then(m => m.AgentconnectorPullComponent)
}, {
    path: 'agentconnector/push',
    loadComponent: () => import('./pages/agentconnector/agentconnector-push/agentconnector-push.component').then(m => m.AgentconnectorPushComponent)
}, {
    path: 'agentconnector/showOutput/:id/:mode',
    loadComponent: () => import('./pages/agentconnector/agentconnector-show-output/agentconnector-show-output.component').then(m => m.AgentconnectorShowOutputComponent)
}, {
    path: 'error/403',
    loadComponent: () => import('./layouts/coreui/errors/error403/error403.component').then(m => m.Error403Component)
}, {
    path: 'error/404',
    loadComponent: () => import('./layouts/coreui/errors/error404/error404.component').then(m => m.Error404Component)
}, {
    path: '**',
    loadComponent: () => import('./layouts/coreui/errors/error404/error404.component').then(m => m.Error404Component)
}
];
export const routes: Routes = [
    ...moduleRoutes,
    ...coreRoutes
];
//}, {
//    path: '**', // TBD: wild card, of custom route matcher for angularjs legacy routes.
//    resolve: {
//        legacyUrl: (route: ActivatedRouteSnapshot, state: RouterStateSnapshot) => {
//            const legacyBaseUrl: string = inject(LEGACY_BASE_URL);
//            const legacyPath: string = state.url;
//            return `${legacyBaseUrl}${legacyPath}`;
//        }
//    },
//    component: LegacyUrlComponent,
//}];
