import {
    ChangeDetectionStrategy,
    ChangeDetectorRef,
    Component,
    EventEmitter,
    inject,
    Input,
    OnDestroy,
    Output,
    ViewChild
} from '@angular/core';
import {
    ButtonCloseDirective,
    ColComponent,
    FormCheckComponent,
    FormCheckInputDirective,
    FormCheckLabelDirective,
    FormControlDirective,
    FormLabelDirective,
    FormSelectDirective,
    FormTextDirective,
    ModalBodyComponent,
    ModalComponent,
    ModalFooterComponent,
    ModalHeaderComponent,
    ModalService,
    ModalTitleDirective,
    ProgressComponent,
    RowComponent
} from '@coreui/angular';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { NgIf } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { RequiredIconComponent } from '../../required-icon/required-icon.component';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { ExternalCommandsService, HostProcessCheckResultItem, } from '../../../services/external-commands.service';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { Subscription } from 'rxjs';
import { HttpErrorResponse } from '@angular/common/http';


@Component({
    selector: 'oitc-hosts-process-checkresult-modal',
    imports: [
        ButtonCloseDirective,
        ColComponent,
        FaIconComponent,
        FormCheckComponent,
        FormCheckInputDirective,
        FormCheckLabelDirective,
        FormControlDirective,
        FormLabelDirective,
        FormSelectDirective,
        FormTextDirective,
        ModalBodyComponent,
        ModalComponent,
        ModalFooterComponent,
        ModalHeaderComponent,
        ModalTitleDirective,
        NgIf,
        ReactiveFormsModule,
        RequiredIconComponent,
        RowComponent,
        TranslocoDirective,
        TranslocoPipe,
        XsButtonDirective,
        FormsModule,
        ProgressComponent
    ],
    templateUrl: './hosts-process-checkresult-modal.component.html',
    styleUrl: './hosts-process-checkresult-modal.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class HostsProcessCheckresultModalComponent implements OnDestroy {
    @Input({required: true}) public items: HostProcessCheckResultItem[] = [];
    @Output() completed = new EventEmitter<boolean>();
    public error: boolean = false;

    public state: 0 | 1 | 2 = 0;

    private readonly TranslocoService = inject(TranslocoService);
    private readonly modalService = inject(ModalService);
    private readonly notyService = inject(NotyService);
    private readonly ExternalCommandsService = inject(ExternalCommandsService);
    private subscriptions: Subscription = new Subscription();
    @ViewChild('modal') private modal!: ModalComponent;
    public processCheckresultModal = {
        state: 0,
        output: this.TranslocoService.translate('Test alert'),
        force_hardstate: false
    };

    public percentage: number = 0;
    public isSending: boolean = false;

    private cdr = inject(ChangeDetectorRef);

    public hideModal() {
        this.isSending = false;
        this.percentage = 0;
        this.error = false;

        this.cdr.markForCheck();

        this.modalService.toggle({
            show: false,
            id: 'hostProcessCheckresultModal'
        });
    }

    public setExternalCommands() {
        if (this.processCheckresultModal.output === '') {
            this.error = true;
            return;
        }


        this.items.forEach((element: HostProcessCheckResultItem) => {
            element.status_code = this.processCheckresultModal.state;
            element.plugin_output = this.processCheckresultModal.output;
            element.forceHardstate = this.processCheckresultModal.force_hardstate
        });


        this.percentage = 0;
        this.isSending = true;
        this.cdr.markForCheck();
        let count = this.items.length;
        let responseCount: number = 0
        let issueCount: number = 0;

        for (let i in this.items) {
            const item = this.items[i];

            this.ExternalCommandsService.process_check_result(item).subscribe({
                next: (value: any) => {
                    responseCount++
                    this.percentage = Math.round((responseCount / count) * 100);
                    this.cdr.markForCheck();

                    if (responseCount === count && issueCount === 0) {
                        // The timeout is not necessary. It is just to show the progress bar at 100% for a short time.
                        setTimeout(() => {
                            // All records have been sent successfully. Reset the modal
                            this.isSending = false;
                            this.percentage = 0;

                            this.hideModal();
                        }, 250);

                        const title: string = this.TranslocoService.translate('Check result added');
                        const msg: string = this.TranslocoService.translate('Commands added successfully to queue');

                        this.notyService.genericSuccess(msg, title);

                        setTimeout(() => {
                            this.completed.emit(true);
                        }, 5000);
                    }

                },
                error: (error: HttpErrorResponse) => {
                    this.cdr.markForCheck();
                    issueCount++

                    responseCount++;
                    this.percentage = Math.round((responseCount / count) * 100);

                    if (responseCount === count && issueCount > 0) {
                        // The timeout is not necessary. It is just to show the progress bar at 100% for a short time.
                        setTimeout(() => {
                            this.isSending = false;
                            this.percentage = 0;
                        }, 250);

                        this.notyService.genericError();

                        this.completed.emit(false);
                    }
                }
            });
        }
    }

    public ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }
}
