import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, Input, OnDestroy, OnInit } from '@angular/core';
import { FormCheckComponent, FormCheckInputDirective, FormCheckLabelDirective } from '@coreui/angular';
import { TranslocoDirective } from '@jsverse/transloco';
import { Subscription } from 'rxjs';
import { SelectionServiceService } from './selection-service.service';
import { FormsModule } from '@angular/forms';

@Component({
    selector: 'oitc-select-all',
    imports: [
        FormCheckComponent,
        FormCheckInputDirective,
        FormCheckLabelDirective,
        TranslocoDirective,
        FormsModule
    ],
    templateUrl: './select-all.component.html',
    styleUrl: './select-all.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class SelectAllComponent implements OnInit, OnDestroy {

    @Input() public itemsCount: number = 0;

    public currentSelection: any[] = [];
    public checked = false;
    public intermediate = false;

    private subscriptions: Subscription = new Subscription();
    private cdr = inject(ChangeDetectorRef);


    constructor(public selection: SelectionServiceService) {
    }

    public ngOnInit() {
        // Subscribe to selection changes
        this.subscriptions.add(this.selection.selection$.subscribe(selection => {
            this.cdr.markForCheck();

            this.currentSelection = selection; // Used to show the length of the selection in the template

            if (selection.length === 0) {
                // 0 items are selected
                // Reset the checked and intermediate state
                this.checked = false;
                this.intermediate = false;
                return;
            }


            if (selection.length === this.itemsCount) {
                // All items are selected
                this.checked = true;
            }

            this.intermediate = false;
            if (selection.length > 0 && selection.length < this.itemsCount) {
                this.intermediate = true;
                this.checked = false;
            }
        }));
    }

    public ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }

    // Called by (change) - no manual change detection required
    public toggleSelectAll() {
        if (this.checked) {
            this.selection.selectAll();
        } else {
            this.selection.deselectAll();
        }
    }

}
