import { ContextActionType, MapItemType } from './map-item-base.enum';
import { Map } from '../../pages/maps/maps.interface';

/********************************
 * Item Base
 ********************************/

export interface MapitemBase {
    id: number
    map_id: number
    x: number
    y: number
    z_index?: string
    display?: boolean
    startX?: number
    startY?: number
    endX?: number
    endY?: number
}

export interface ContextAction {
    type: ContextActionType
    data: MapitemBase
    itemType: MapItemType
    item?: any // to send the item for edit
}

export interface MapitemBaseActionObject {
    data: MapitemBase
    action: string
    type: MapItemType
}

export interface ResizedEvent {
    id: number
    mapId: number
    itemType: MapItemType
    width: number
    height: number
}


/********************************
 * Items
 ********************************/


export interface MapItemRootParams {
    angular: true,
    disableGlobalLoader: true,
    objectId: number,
    mapId: number,
    type: string,
    includeServiceOutput?: true
}

export interface MapItemRoot {
    type?: string
    data: DataForMapItem
    _csrfToken?: string
}

export interface DataForMapItem {
    BitMaskHostState?: number
    BitMaskServiceState?: number
    icon?: string
    icon_property?: string
    isAcknowledged?: boolean
    isInDowntime?: boolean
    color?: string
    background?: string
    Host?: HostForMapItem | HostForMapsummaryItem
    Hoststatus?: HoststatusForMapItem
    Service?: ServiceForMapItem | ServiceForMapsummaryItem
    Perfdata?: Perfdata
    Servicestatus: ServicestatusForMapItem
    Hostgroup?: HostgroupForMapItem
    Servicegroup?: ServicegroupForMapItem
    Map?: Map
    hostname?: string
    servicename?: string
}

export interface HostForMapItem {
    id: number
    uuid: string
    display_name: string
    type_id: number
}

export interface HostForMapsummaryItem {
    currentState: number
    isFlapping: boolean
    problemHasBeenAcknowledged: boolean
    scheduledDowntimeDepth: number
    lastCheck: number
    nextCheck: number
    activeChecksEnabled: boolean
    lastHardState: number
    lastHardStateChange: number
    last_state_change: number
    output: string
    long_output: string
    acknowledgement_type: number
    state_type: boolean
    flap_detection_enabled: boolean
    notifications_enabled: boolean
    current_check_attempt: number
    max_check_attempts: number
    latency: number
    last_time_up: number
    UserTime: any
    isHardstate: boolean
    isInMonitoring: boolean
    humanState: string
    cssClass: string
    textClass: string
    outputHtml: string
}

export interface HoststatusForMapItem {
    currentState: number
    isFlapping: boolean
    problemHasBeenAcknowledged: boolean
    scheduledDowntimeDepth: number
    lastCheck: string
    nextCheck: string
    activeChecksEnabled: number
    lastHardState: any
    lastHardStateChange: string
    last_state_change: string
    output: any
    long_output: any
    perfdata?: string
    acknowledgement_type: any
    state_type: any
    flap_detection_enabled: number
    notifications_enabled: number
    current_check_attempt: any
    max_check_attempts: number
    latency: any
    last_time_up: string
    UserTime?: any
    lastHardStateChangeInWords?: string
    last_state_change_in_words?: string
    lastCheckInWords?: string
    nextCheckInWords?: string
    isHardstate: boolean
    isInMonitoring: boolean
    humanState: string
    cssClass: string
    textClass: string
    outputHtml: string
}

export interface ServiceForMapItem {
    id: number
    uuid: string
    display_name: string
    type_id: number
    servicename?: string
    disabled?: boolean
}

export interface ServiceForMapsummaryItem {
    currentState: number
    lastHardState: number
    isFlapping: boolean
    problemHasBeenAcknowledged: boolean
    scheduledDowntimeDepth: number
    lastCheck: number
    nextCheck: number
    activeChecksEnabled: boolean
    lastHardStateChange: number
    last_state_change: number
    processPerformanceData: boolean
    state_type: boolean
    acknowledgement_type: number
    flap_detection_enabled: boolean
    notifications_enabled: boolean
    current_check_attempt: number
    output: string
    long_output: string
    perfdata: string
    latency: number
    max_check_attempts: number
    last_time_ok: number
    UserTime: any
    isHardstate: boolean
    isInMonitoring: boolean
    humanState: string
    cssClass: string
    textClass: string
    outputHtml: string
}

export interface Perfdata {
    [key: string]: PerformanceData
}

export interface PerformanceData {
    current: string
    unit: string
    warning: string
    critical: string
    min: string
    max: string
    metric?: string
    datasource?: Datasource
    prometheus?: Prometheus
}

export interface Prometheus {
    instance: string
    job: string
    service: string
    uuid: string
}

export interface Datasource {
    setup: Setup
}

export interface Setup {
    metric: Metric
    scale: Scale
    warn: Threshold
    crit: Threshold
}

export interface Metric {
    value: number
    unit: string
    name: string
}

export interface Scale {
    min: number
    max: number
    type: string
}

export interface Threshold {
    low: number | null
    high: any
}

export interface ServicestatusForMapItem {
    currentState: number
    lastHardState: any
    isFlapping: boolean
    problemHasBeenAcknowledged: boolean
    scheduledDowntimeDepth: number
    lastCheck: string
    nextCheck: string
    activeChecksEnabled: any
    lastHardStateChange: string
    last_state_change: string
    processPerformanceData: any
    state_type: any
    acknowledgement_type: any
    flap_detection_enabled: any
    notifications_enabled: any
    current_check_attempt: number
    output: string
    long_output: any
    perfdata: string
    latency: any
    max_check_attempts: number
    last_time_ok: string
    UserTime?: any
    lastHardStateChangeInWords?: string
    last_state_change_in_words?: string
    lastCheckInWords?: string
    nextCheckInWords?: string
    isHardstate: boolean
    isInMonitoring: boolean
    humanState: string
    cssClass: string
    textClass: string
    outputHtml: string
    longOutputHtml?: string
}

export interface HostgroupForMapItem {
    uuid: string
    name: string
    description: string
}

export interface ServicegroupForMapItem {
    uuid: string
    name: string
    description: string
}


