import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import { AgentconnectorService } from '../agentconnector.service';
import { AgentconnectorPullRoot } from '../agentconnector.interface';
import { Subscription } from 'rxjs';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { TranslocoDirective, TranslocoPipe } from '@jsverse/transloco';
import {
    AlertComponent,
    CardBodyComponent,
    CardComponent,
    CardFooterComponent,
    CardHeaderComponent,
    CardTitleDirective,
    NavComponent,
    NavItemComponent,
    TableDirective
} from '@coreui/angular';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { AsyncPipe, NgIf } from '@angular/common';
import { TableLoaderComponent } from '../../../layouts/primeng/loading/table-loader/table-loader.component';
import { RouterLink } from '@angular/router';
import { NoRecordsComponent } from '../../../layouts/coreui/no-records/no-records.component';
import { PermissionsService } from '../../../permissions/permissions.service';

@Component({
    selector: 'oitc-agentconnector-pull',
    imports: [
        FaIconComponent,
        PermissionDirective,
        TranslocoDirective,
        CardComponent,
        CardHeaderComponent,
        CardTitleDirective,
        NavComponent,
        NavItemComponent,
        XsButtonDirective,
        CardBodyComponent,
        CardFooterComponent,
        NgIf,
        TableLoaderComponent,
        RouterLink,
        TableDirective,
        AsyncPipe,
        TranslocoPipe,
        NoRecordsComponent,
        AlertComponent
    ],
    templateUrl: './agentconnector-pull.component.html',
    styleUrl: './agentconnector-pull.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class AgentconnectorPullComponent implements OnInit, OnDestroy {

    public agents?: AgentconnectorPullRoot;

    public readonly PermissionsService = inject(PermissionsService);

    private readonly AgentconnectorService = inject(AgentconnectorService)
    private cdr = inject(ChangeDetectorRef);
    private subscriptions: Subscription = new Subscription();

    public ngOnInit(): void {
        this.loadPullAgents();
    }

    public ngOnDestroy(): void {
        this.subscriptions.unsubscribe();
    }

    public loadPullAgents() {

        this.subscriptions.add(this.AgentconnectorService.getPullAgents()
            .subscribe((result) => {
                this.agents = result;
                this.cdr.markForCheck();
            })
        );
    }
}
