import { PaginateOrScroll } from '../../layouts/coreui/paginator/paginator.interface';

export enum AgentModes {
    Pull = 'pull',
    Push = 'push'
}

/************************
 * Agentconnector Pull  *
 ***********************/
export interface AgentconnectorPullRoot {
    agents: PullAgent[]
    _csrfToken: string
}

export interface PullAgent {
    port: number
    use_https: boolean
    insecure: boolean
    use_autossl: boolean
    use_push_mode: false
    basic_auth: boolean
    host: {
        uuid: string
        name: string
        address: string
    }
}

/************************
 * Agentconnector Push  *
 ***********************/

export interface AgentconnectorPushRoot extends PaginateOrScroll {
    agents: PushAgent[]
    _csrfToken: string
}

export interface PushAgent {
    id: number
    uuid: string
    host_uuid: null | string
    hostname: string
    ipaddress: string
    remote_address: string
    http_x_forwarded_for: null | string
    last_update: string
    HostMapping: {
        uuid: null | string
        display_name: null | string
    }
}

export interface AgentsPushParams {
    angular: true,
    scroll: boolean,
    sort: string,
    page: number,
    direction: 'asc' | 'desc' | ''
    'filter[HostMapping.display_name]': string
    'filter[hasHostAssignment]': string | boolean
}

export function getDefaultAgentsPushParams(): AgentsPushParams {
    return {
        angular: true,
        scroll: true,
        sort: 'HostMapping.display_name',
        page: 1,
        direction: 'asc',
        'filter[HostMapping.display_name]': '',
        'filter[hasHostAssignment]': '',
    }
}


/**********************
 * showOutput action  *
 **********************/
export interface AgentconnectorShowOutputParams {
    angular: true,
    id: number, // hostId or push_agents.id in push mode
    mode: AgentModes
}

export interface AgentconnectorShowOutputRoot {
    outputAsJson: string
    agent: {
        uuid: string
        ipaddress: string,
        remote_address: string,
        http_x_forwarded_for: null | string
    }
    _csrfToken: string
}

