import { inject, Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { PROXY_PATH } from '../../tokens/proxy-path.token';
import { map, Observable } from 'rxjs';
import {
    AgentconnectorPullRoot,
    AgentconnectorPushRoot,
    AgentconnectorShowOutputParams,
    AgentconnectorShowOutputRoot,
    AgentsPushParams
} from './agentconnector.interface';
import { DeleteAllItem } from '../../layouts/coreui/delete-all-modal/delete-all.interface';

@Injectable({
    providedIn: 'root'
})
export class AgentconnectorService {
    private readonly http = inject(HttpClient);
    private readonly proxyPath = inject(PROXY_PATH);

    constructor() {
    }

    public getPullAgents(): Observable<AgentconnectorPullRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<AgentconnectorPullRoot>(`${proxyPath}/agentconnector/pull.json`, {
            params: {
                angular: true
            }
        }).pipe(
            map(data => {
                return data;
            })
        )
    }

    public getPushAgents(params: AgentsPushParams): Observable<AgentconnectorPushRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<AgentconnectorPushRoot>(`${proxyPath}/agentconnector/push.json`, {
            params: params as {}
        }).pipe(
            map(data => {
                return data;
            })
        )
    }


    // Generic function for the Delete All Modal
    public delete(item: DeleteAllItem): Observable<Object> {
        const proxyPath = this.proxyPath;
        return this.http.post(`${proxyPath}/agentconnector/delete/${item.id}.json?angular=true`, {});
    }

    public showOutput(params: AgentconnectorShowOutputParams): Observable<AgentconnectorShowOutputRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<AgentconnectorShowOutputRoot>(`${proxyPath}/agentconnector/showOutput.json`, {
            params: params as {}
        }).pipe(
            map(data => {
                return data;
            })
        )
    }
}
