import { PaginateOrScroll } from '../../layouts/coreui/paginator/paginator.interface';

export interface DowntimeObject {
    hostname: string //uuid
    service_description?: string // uuid

    author_name: string
    comment_data: string
    entry_time: string
    scheduled_start_time: string
    scheduled_end_time: string
    duration: number
    was_started: boolean
    internal_downtime_id: number
    was_cancelled: boolean
    duration_for_humans: string
    is_cancellable: boolean
    is_running: boolean
    is_expired: boolean
}

export interface HostBrowserDowntime {
    author_name: string
    comment_data: string
    entry_time: string
    scheduled_start_time: string
    scheduled_end_time: string
    duration: number
    was_started: boolean
    internal_downtime_id: number
    was_cancelled: boolean
    duration_for_humans: string
    is_cancellable: boolean
    is_running: boolean
    is_expired: boolean
}

export interface ServiceBrowserDowntime {
    hostname: string
    service_description: string
    author_name: string
    comment_data: string
    entry_time: string
    scheduled_start_time: string
    scheduled_end_time: string
    duration: number
    was_started: boolean
    internal_downtime_id: number
    was_cancelled: boolean
    duration_for_humans: string
    is_cancellable: boolean
    is_running: boolean
    is_expired: boolean
}

/**********************
 *    Host action    *
 **********************/
export interface DowntimesHostParams {
    angular: true,
    scroll: boolean,
    sort: string,
    page: number,
    direction: 'asc' | 'desc' | '', // asc or desc
    'filter[DowntimeHosts.hostname]': string, //uuid
    'filter[DowntimeHosts.comment_data]': string,
    'filter[DowntimeHosts.author_name]': string,
    'filter[DowntimeHosts.was_cancelled]': string | number,
    'filter[hideExpired]': boolean,
    'filter[isRunning]': boolean,
    'filter[from]': Date | string,
    'filter[to]': Date | string,
}

export function getDefaultDowntimesHostParamsParams(): DowntimesHostParams {
    let now = new Date();
    return {
        angular: true,
        scroll: true,
        sort: 'DowntimeHosts.scheduled_start_time',
        page: 1,
        direction: 'desc',
        'filter[DowntimeHosts.hostname]': '',
        'filter[DowntimeHosts.comment_data]': '',
        'filter[DowntimeHosts.author_name]': '',
        'filter[DowntimeHosts.was_cancelled]': '',
        'filter[hideExpired]': true,
        'filter[isRunning]': false,
        'filter[from]': new Date(now.getTime() - (3600 * 24 * 1000 * 30)),
        'filter[to]': new Date(now.getTime() + (3600 * 24 * 30 * 1000 * 2))
    }
}

export interface DowntimesHostRoot extends PaginateOrScroll {
    all_host_downtimes: DowntimeObject[]
}

/**********************
 *   Service action   *
 **********************/
export interface DowntimesServiceParams {
    angular: true,
    scroll: boolean,
    sort: string,
    page: number,
    direction: 'asc' | 'desc' | '', // asc or desc
    'filter[DowntimeServices.service_description]': string, //uuid
    'filter[DowntimeServices.comment_data]': string,
    'filter[DowntimeServices.author_name]': string,
    'filter[DowntimeServices.was_cancelled]': string | number,
    'filter[hideExpired]': boolean,
    'filter[isRunning]': boolean,
    'filter[from]': Date | string,
    'filter[to]': Date | string,
}

export function getDefaultDowntimesServiceParamsParams(): DowntimesServiceParams {
    let now = new Date();
    return {
        angular: true,
        scroll: true,
        sort: 'DowntimeServices.scheduled_start_time',
        page: 1,
        direction: 'desc',
        'filter[DowntimeServices.service_description]': '',
        'filter[DowntimeServices.comment_data]': '',
        'filter[DowntimeServices.author_name]': '',
        'filter[DowntimeServices.was_cancelled]': '',
        'filter[hideExpired]': true,
        'filter[isRunning]': false,
        'filter[from]': new Date(now.getTime() - (3600 * 24 * 1000 * 30)),
        'filter[to]': new Date(now.getTime() + (3600 * 24 * 30 * 1000 * 2))
    }
}

export interface DowntimesServiceRoot extends PaginateOrScroll {
    all_service_downtimes: DowntimeObject[],
    hostUuid: string
}
