import { PaginateOrScroll } from '../../layouts/coreui/paginator/paginator.interface';
import { AcknowledgementTypes } from '../acknowledgements/acknowledgement-types.enum';
import { HostBrowserAcknowledgement } from '../acknowledgements/acknowledgement.interface';
import { HostBrowserDowntime } from '../downtimes/downtimes.interface';

/**********************
 *    Index action    *
 **********************/
export interface HostsIndexRoot extends PaginateOrScroll {
    hoststatus: HoststatusIndex[]
    username: string
}

export interface Hoststatus {
    hostname: string
    status_update_time: string
    output: string
    long_output: string
    perfdata: string
    current_state: number
    current_check_attempt: number
    max_check_attempts: number
    last_check: string
    next_check: string
    is_passive_check: boolean
    last_state_change: string
    last_hard_state_change: string
    last_hard_state: number
    is_hardstate: boolean
    last_notification: number
    next_notification: number
    notifications_enabled: boolean
    problem_has_been_acknowledged: boolean
    acknowledgement_type: AcknowledgementTypes
    passive_checks_enabled: boolean
    active_checks_enabled: boolean
    event_handler_enabled: boolean
    flap_detection_enabled: boolean
    is_flapping: boolean
    latency: number
    execution_time: number
    scheduled_downtime_depth: number
    process_performance_data: boolean
    obsess_over_host: boolean
    normal_check_interval: number
    retry_check_interval: number
    check_timeperiod: string
    node_name: string
    last_time_up: string
    last_time_down: string
    last_time_unreachable: string
    current_notification_number: number
    percent_state_change: number
    event_handler: any | string
    check_command: string
    host_mapping: HostMapping
    servicestatus_summary: ServicestatusSummary
    last_check_in_words: string
    next_check_in_words: string
    last_state_change_for_humans: string
    last_hard_state_change_for_humans: string
    normal_check_interval_for_humans: string
    retry_check_interval_for_humans: string
    output_html: string
    long_output_html: string
}

export interface HoststatusIndex extends Hoststatus {
    servicestatus_summary: ServicestatusSummary
}

export interface HoststatusBrowser extends Hoststatus {
    timeperiod_mapping: TimeperiodMapping

}

export interface HostsBrowserRoot {
    hoststatus: HoststatusBrowser
    acknowledgement: HostBrowserAcknowledgement | null
    downtime: HostBrowserDowntime | null
    username: string
    _csrfToken: string
}

export interface HostMapping {
    id: number
    uuid: string
    display_name: string
    type_id: number
}

export interface TimeperiodMapping {
    id: number
    uuid: string
    display_name: string
    type_id: number
}

export interface ServicestatusSummary {
    ok: number
    warning: number
    critical: number
    unknown: number
}

export interface HostsIndexParams {
    angular: true,
    scroll: boolean,
    sort: string,
    page: number,
    direction: 'asc' | 'desc' | ''
    'filter[current_state][]': string[], // up, down, unreachable
    'filter[HostMapping.display_name]': string
    'filter[output]': string
    'filter[Hoststatus.problem_has_been_acknowledged]': string | boolean,
    'filter[Hoststatus.scheduled_downtime_depth]': string | boolean,
    'filter[Hoststatus.is_hardstate]': string | boolean
}

export function getDefaultHostsIndexParams(): HostsIndexParams {
    return {
        angular: true,
        scroll: true,
        sort: 'Hoststatus.current_state',
        page: 1,
        direction: 'desc',
        'filter[current_state][]': [],
        'filter[HostMapping.display_name]': '',
        'filter[output]': '',
        'filter[Hoststatus.problem_has_been_acknowledged]': '',
        'filter[Hoststatus.scheduled_downtime_depth]': '',
        'filter[Hoststatus.is_hardstate]': ''
    }
}

export interface HostsCurrentStateFilter {
    up: boolean
    down: boolean
    unreachable: boolean
}

export function getHostCurrentStateForApi(currentState: HostsCurrentStateFilter): string[] {
    let result = [];
    if (currentState.up) {
        result.push('up');
    }
    if (currentState.down) {
        result.push('down');
    }
    if (currentState.unreachable) {
        result.push('unreachable');
    }

    return result;
}
