import {
    ChangeDetectionStrategy,
    ChangeDetectorRef,
    Component,
    EventEmitter,
    inject,
    OnDestroy,
    OnInit,
    Output
} from '@angular/core';
import {
    AlertComponent,
    BorderDirective,
    ButtonCloseDirective,
    ButtonGroupComponent,
    ButtonToolbarComponent,
    CardBodyComponent,
    CardComponent,
    CardHeaderComponent,
    ColComponent,
    ModalBodyComponent,
    ModalComponent,
    ModalFooterComponent,
    ModalHeaderComponent,
    ModalService,
    ModalTitleDirective,
    RowComponent,
    TooltipDirective
} from '@coreui/angular';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import {
    HostsDisableFlapdetectionModalComponent
} from '../../../../components/hosts/hosts-disable-flapdetection-modal/hosts-disable-flapdetection-modal.component';
import {
    HostsEnableFlapdetectionModalComponent
} from '../../../../components/hosts/hosts-enable-flapdetection-modal/hosts-enable-flapdetection-modal.component';
import { AsyncPipe, NgClass, NgIf } from '@angular/common';
import {
    ServiceAcknowledgeModalComponent
} from '../../../../components/services/service-acknowledge-modal/service-acknowledge-modal.component';
import {
    ServiceMaintenanceModalComponent
} from '../../../../components/services/service-maintenance-modal/service-maintenance-modal.component';
import {
    ServiceResetChecktimeModalComponent
} from '../../../../components/services/service-reset-checktime-modal/service-reset-checktime-modal.component';
import { ServiceStatusNamePipe } from '../../../../pipes/service-status-name.pipe';
import {
    ServicesProcessCheckresultModalComponent
} from '../../../../components/services/services-process-checkresult-modal/services-process-checkresult-modal.component';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { XsButtonDirective } from '../../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { ServicesBrowserRoot, ServicestatusBrowser } from '../../services.interface';
import { HostsBrowserRoot } from '../../../hosts/hosts.interface';
import {
    ExternalCommandsService,
    ServiceProcessCheckResultItem,
    ServiceResetItem
} from '../../../../services/external-commands.service';
import { ExternalCommandsEnum } from '../../../../enums/external-commands.enum';
import { HostBrowserDowntime, ServiceBrowserDowntime } from '../../../downtimes/downtimes.interface';
import { CancelAllItem } from '../../../downtimes/cancel-hostdowntime-modal/cancel-hostdowntime.interface';
import { TimezoneConfiguration as TimezoneObject, TimezoneService } from '../../../../services/timezone.service';
import { SelectKeyValueString } from '../../../../layouts/primeng/select.interface';
import { BehaviorSubject, Subscription } from 'rxjs';
import { NotyService } from '../../../../layouts/coreui/noty.service';
import { ActivatedRoute, Router, RouterLink } from '@angular/router';
import { HostsService } from '../../../hosts/hosts.service';
import { ServicesService } from '../../services.service';
import { PermissionsService } from '../../../../permissions/permissions.service';
import { HistoryService } from '../../../../history.service';
import { DowntimesService } from '../../../downtimes/downtimes.service';
import { AcknowledgementsService } from '../../../acknowledgements/acknowledgements.service';
import { BlockLoaderComponent } from '../../../../layouts/primeng/loading/block-loader/block-loader.component';
import { TrustAsHtmlPipe } from '../../../../pipes/trust-as-html.pipe';
import { DELETE_SERVICE_TOKEN } from '../../../../tokens/delete-injection.token';
import { ServicesBrowserChartComponent } from '../services-browser-chart/services-browser-chart.component';


import { PermissionDirective } from '../../../../permissions/permission.directive';
import {
    CancelHostdowntimeModalComponent
} from '../../../downtimes/cancel-hostdowntime-modal/cancel-hostdowntime-modal.component';
import {
    CancelServicedowntimeModalComponent
} from '../../../downtimes/cancel-servicedowntime-modal/cancel-servicedowntime-modal.component';
import { ServiceBrowserModalService } from './service-browser-modal.service';
import { GenericUnixtimerange } from '../../../../generic.interfaces';

@Component({
    selector: 'oitc-service-browser-modal',
    imports: [
        ButtonCloseDirective,
        ButtonGroupComponent,
        ButtonToolbarComponent,
        ColComponent,
        FaIconComponent,
        HostsDisableFlapdetectionModalComponent,
        HostsEnableFlapdetectionModalComponent,
        ModalBodyComponent,
        ModalComponent,
        ModalFooterComponent,
        ModalHeaderComponent,
        ModalTitleDirective,
        NgIf,
        RowComponent,
        ServiceAcknowledgeModalComponent,
        ServiceMaintenanceModalComponent,
        ServiceResetChecktimeModalComponent,
        ServiceStatusNamePipe,
        ServicesProcessCheckresultModalComponent,
        TranslocoPipe,
        XsButtonDirective,
        TranslocoDirective,
        NgClass,
        BlockLoaderComponent,
        TooltipDirective,
        TrustAsHtmlPipe,
        ServicesBrowserChartComponent,
        AlertComponent,
        CardBodyComponent,
        CardComponent,
        CardHeaderComponent,
        PermissionDirective,
        BorderDirective,
        RouterLink,
        CancelHostdowntimeModalComponent,
        CancelServicedowntimeModalComponent,
        AsyncPipe
    ],
    templateUrl: './service-browser-modal.component.html',
    styleUrl: './service-browser-modal.component.css',
    providers: [
        {provide: DELETE_SERVICE_TOKEN, useClass: DowntimesService}, // Inject the DowntimesService into the CancelAllModalComponent
    ],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class ServiceBrowserModalComponent implements OnInit, OnDestroy {

    public serviceUuid: string = '';

    @Output() completed: EventEmitter<boolean> = new EventEmitter<boolean>();

    public timezone!: TimezoneObject;
    public result?: ServicesBrowserRoot;
    public hoststatus?: HostsBrowserRoot;

    public availableDataSources: SelectKeyValueString[] = []; // The API result is not as good
    public selectedItems: any[] = [];

    private readonly timerange$$ = new BehaviorSubject<GenericUnixtimerange>({start: 0, end: 0});
    public readonly timerange$ = this.timerange$$.asObservable();

    public cssClass: string = 'bg-unknown';
    public textClass: string = 'unknown';

    private subscriptions: Subscription = new Subscription();
    private readonly HostsService = inject(HostsService);
    private readonly ServicesService = inject(ServicesService);
    private readonly notyService = inject(NotyService);
    private readonly TimezoneService = inject(TimezoneService);
    public readonly PermissionsService = inject(PermissionsService);
    private router = inject(Router);
    private route = inject(ActivatedRoute);
    private readonly HistoryService: HistoryService = inject(HistoryService);
    private readonly ExternalCommandsService = inject(ExternalCommandsService);
    private readonly TranslocoService = inject(TranslocoService);
    private readonly modalService = inject(ModalService);
    private readonly DowntimesService = inject(DowntimesService);
    private readonly AcknowledgementsService = inject(AcknowledgementsService);
    private readonly ServiceBrowserModalService = inject(ServiceBrowserModalService);
    private cdr = inject(ChangeDetectorRef);

    public hideModal() {
        this.modalService.toggle({
            show: false,
            id: 'automapServiceDetailsModal'
        });
    }

    private resetModal(): void {
        // Reset everything
        this.selectedItems = [];
        this.availableDataSources = [];
        this.result = undefined;
        this.cdr.markForCheck();
    }

    public ngOnInit(): void {
        this.getUserTimezone();

        this.subscriptions.add(this.ServiceBrowserModalService.serviceUuid$.subscribe((serviceUuid) => {
            this.serviceUuid = serviceUuid;

            this.resetModal();
            this.loadService();
            this.cdr.markForCheck();

            // open modal
            this.modalService.toggle({
                show: true,
                id: 'automapServiceDetailsModal'

            });
        }));


        //this.subscriptions.add(this.modalService.modalState$.subscribe((state) => {
        //    if (state.id === 'automapServiceDetailsModal') {
        //        this.resetModal();
        //        this.loadService();
        //        this.cdr.markForCheck();
        //    }
        //}));


    }

    public ngOnDestroy(): void {
        this.subscriptions.unsubscribe();
    }

    private getUserTimezone() {
        this.subscriptions.add(this.TimezoneService.getTimezoneConfiguration().subscribe(data => {
            this.timezone = data;
            this.cdr.markForCheck();
        }));
    }

    public loadService() {
        this.subscriptions.add(this.ServicesService.getServiceBrowser(this.serviceUuid).subscribe((result) => {
            this.result = result;

            const hostUuid = result.servicestatus.hostname;

            // Load host status
            this.subscriptions.add(this.HostsService.getHostBrowser(hostUuid).subscribe((result) => {
                this.cdr.markForCheck();
                this.hoststatus = result;
            }));

            switch (result.servicestatus.current_state) {
                case 0:
                    this.cssClass = 'bg-ok';
                    this.textClass = 'ok';
                    break;
                case 1:
                    this.cssClass = 'bg-warning';
                    this.textClass = 'warning';
                    break;
                case 2:
                    this.cssClass = 'bg-critical';
                    this.textClass = 'critical';
                    break;

                default:
                    this.cssClass = 'bg-unknown';
                    this.textClass = 'unknown';
                    break;
            }

            for (let key in result.Perfdata) {
                const gauge = result.Perfdata[key];
                this.availableDataSources.push({
                    key: key, // load this datasource
                    value: key // Name of the metric to display in select
                });
            }
            this.cdr.markForCheck();
        }));
    }

    public resetChecktime(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        const commands: ServiceResetItem[] = [
            {
                command: ExternalCommandsEnum.rescheduleService,
                hostUuid: servicestatus.hostname,
                serviceUuid: servicestatus.service_description,
            }
        ];

        if (commands.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }

        commands.forEach(command => {
            this.subscriptions.add(this.ExternalCommandsService.reset_check_time(command).subscribe((result) => {
            }));
        });

        const title = this.TranslocoService.translate('Reschedule');
        const msg: string = this.TranslocoService.translate('Commands added successfully to queue');

        this.notyService.genericSuccess(msg, title);

        setTimeout(() => {
            this.onMassActionComplete(true);
        }, 5000);
    }

    public toggleDowntimeModal(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        if (this.result) {
            this.selectedItems.push({
                command: ExternalCommandsEnum.submitServiceDowntime,
                hostUuid: servicestatus.hostname,
                serviceUuid: servicestatus.service_description,
                start: 0,
                end: 0,
                author: this.result.username,
                comment: '',
            });

            this.modalService.toggle({
                show: true,
                id: 'serviceMaintenanceModal',
            });
        }
    }

    public acknowledgeStatus(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        if (this.result) {
            this.selectedItems.push({
                command: ExternalCommandsEnum.submitServicestateAck,
                hostUuid: servicestatus.hostname,
                serviceUuid: servicestatus.service_description,
                sticky: 0,
                notify: false,
                author: this.result.username,
                comment: '',
            });

            this.modalService.toggle({
                show: true,
                id: 'serviceAcknowledgeModal',
            });
        }

        setTimeout(() => {
            this.loadService()
        }, 5000);
    }

    public processCheckResult(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        const item: ServiceProcessCheckResultItem = {
            command: ExternalCommandsEnum.commitPassiveServiceResult,
            maxCheckAttempts: servicestatus.max_check_attempts,
            hostUuid: servicestatus.hostname,
            serviceUuid: servicestatus.service_description,
            status_code: 0, // Will be overwritten with the modals default value
            plugin_output: '', // Will be overwritten with the modals default value
            forceHardstate: false, // Will be overwritten with the modals default value
            long_output: ''
        };

        this.selectedItems.push(item);

        this.modalService.toggle({
            show: true,
            id: 'serviceProcessCheckresultModal',
        });
    }

    public enableFlapdetection(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        this.selectedItems.push({
            command: ExternalCommandsEnum.enableOrDisableServiceFlapdetection,
            hostUuid: servicestatus.hostname,
            serviceUuid: servicestatus.service_description,
            condition: 1
        });

        this.modalService.toggle({
            show: true,
            id: 'hostEnableFlapdetectionModal',
        });
    }

    public disableFlapdetection(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        this.selectedItems.push({
            command: ExternalCommandsEnum.enableOrDisableServiceFlapdetection,
            hostUuid: servicestatus.hostname,
            serviceUuid: servicestatus.service_description,
            condition: 0
        });

        this.modalService.toggle({
            show: true,
            id: 'hostDisableFlapdetectionModal',
        });
    }

    public toggleCancelHostDowntimeModal(hostDowntime: HostBrowserDowntime) {
        this.selectedItems = [];

        const item: CancelAllItem[] = [{
            id: hostDowntime.internal_downtime_id
        }];

        // Pass selection to the modal
        this.selectedItems = item;

        // open modal
        this.modalService.toggle({
            show: true,
            id: 'cancelHostAllModal',
        });
    }

    public toggleCancelServiceDowntimeModal(serviceDowntime: ServiceBrowserDowntime) {
        this.selectedItems = [];

        const item: CancelAllItem[] = [{
            id: serviceDowntime.internal_downtime_id
        }];

        // Pass selection to the modal
        this.selectedItems = item;

        // open modal
        this.modalService.toggle({
            show: true,
            id: 'cancelServiceAllModal',
        });
    }

    public onToggleRescheduling(event: boolean) {
        // Service browser menu wants to toggle a rescheduling
        if (this.result) {
            this.resetChecktime(this.result.servicestatus);
        }
    }

    public onMassActionComplete(success: boolean) {
        this.completed.emit(success); // Pass the event to the parent component
        if (success) {
            this.loadService();
        }
    }

    protected readonly Number = Number;
    protected readonly String = String;
}
