import { inject, Injectable } from '@angular/core';
import { map, Observable } from 'rxjs';
import { HttpClient } from '@angular/common/http';
import { PROXY_PATH } from '../../tokens/proxy-path.token';
import {
    ServicesAutomapParams,
    ServicesAutomapRoot,
    ServicesBrowserRoot,
    ServicesIndexParams,
    ServicesIndexRoot
} from './services.interface';

@Injectable({
    providedIn: 'root'
})
export class ServicesService {
    private readonly http = inject(HttpClient);
    private readonly proxyPath = inject(PROXY_PATH);

    constructor() {
    }

    public getIndex(params: ServicesIndexParams): Observable<ServicesIndexRoot> {
        const proxyPath = this.proxyPath;

        return this.http.get<ServicesIndexRoot>(`${proxyPath}/services/index.json`, {
            params: params as {}
        }).pipe(
            map(data => {
                return data;
            })
        );
    }

    public getServiceBrowser(serviceUuid: string): Observable<ServicesBrowserRoot> {
        const proxyPath = this.proxyPath;

        return this.http.get<ServicesBrowserRoot>(`${proxyPath}/services/browser.json`, {
            params: {
                serviceuuid: serviceUuid
            }
        }).pipe(
            map(data => {
                return data;
            })
        );
    }

    public getServiceAutomap(params: ServicesAutomapParams): Observable<ServicesAutomapRoot> {
        const proxyPath = this.proxyPath;

        return this.http.get<ServicesAutomapRoot>(`${proxyPath}/services/automap.json`, {
            params: params as {}
        }).pipe(
            map(data => {
                return data;
            })
        );
    }
}
