import { PaginateOrScroll } from '../../layouts/coreui/paginator/paginator.interface';

export interface UsergroupsIndexRoot extends PaginateOrScroll {
    allUsergroups: Usergroup[]
    myUserId: number
    _csrfToken: string
}

export interface Usergroup {
    id?: number
    name: string
    description: string
    usercount: number
}

export interface UsergroupWithoutUsercount {
    id?: number
    name: string
    description: string
}

export interface UsergroupsIndexParams {
    angular: true
    scroll: boolean
    sort: string
    page: number
    direction: 'asc' | 'desc' | '' // asc or desc

    'filter[Usergroups.description]': string
    'filter[Usergroups.name]': string
}

export function getDefaultUsergroupsIndexParams(): UsergroupsIndexParams {
    return {
        angular: true,
        scroll: true,
        sort: 'Usergroups.name',
        page: 1,
        direction: 'asc',
        'filter[Usergroups.description]': "",
        'filter[Usergroups.name]': ""
    }
}

export interface UsergroupDefault {
    Acos: {
        [key: number]: number
    }
    Usergroup: UsergroupWithoutUsercount
}

export interface UsergroupsEditGet {
    usergroup: UsergroupAro
    acos: {
        [key: number]: number
    }
    _csrfToken: string
}

export interface UsergroupAro {
    acos: { [key: number]: AcoChild }
    created: string
    description: string
    id: number
    modified: string
    name: string
    aro: Aro | undefined
}

export interface Aro {
    id: number
    parent_id: any
    model: string
    foreign_key: number
    alias: any
    lft: number
    rght: number
    acos: { [key: number]: AcoChild }
}

/* ACOs */
export interface Acos {
    acos: { [key: number]: AcoChild }
    _csrfToken: string
}

export interface AcoChild {
    id: number
    parent_id: number
    model: any
    foreign_key: any
    alias: string
    lft: number
    rght: number
    children: { [key: number]: AcoChild }
    _joinData: {
        id: number
        aro_id: number
        aco_id: number
        _create: string
        _read: string
        _update: string
        _delete: string
    }
}
